/***** privbind.c ********/

/* Routines to bind to privileged IP ports by asking privportd. */

/******************************************************************************
 * Copyright (c) 1995,1996 Donated to the public domain                       *
 *                                                                            *
 * Original author and maintainer: Marc Chatel   (mchatel@dial.oleane.com)    *
 * Last known maintainer:          Marc Chatel   (mchatel@dial.oleane.com)    *
 *                                                                            *
 * This file was created as part of the S4 (Secure System Setup Script) kit.  *
 * Permission is granted to any person or entity to do any of the following:  *
 *       a) use this file alone or in some other software                     *
 *       b) modify this file or include parts of this file in other files     *
 *       c) re-distribute this file AS IS or modified, for non-commercial     *
 *          or commercial purposes, alone or as part of some software package *
 *                                                                            *
 * No warranties of any kind, express or implied, on the functionality and    *
 * safety of the contents of this file. Use at your own risk!                 *
 *                                                                            *
 * If you do useful changes to this file (bug fixes, portability fixes,       *
 * enhancements), you should TRY to contact the current maintainer, who may   *
 * be maintaining a "latest greatest" version of the file. You do not HAVE    *
 * TO, but you should TRY. Promote software reuse! It helps everybody,        *
 * including you!                                                             *
 ******************************************************************************/

#include   <standards.h>
#include   <sys/types.h>
#include   <sys/resource.h>
#include   <wait.h>
#include   <stdio.h>
#include   <sys/wait.h>
#include   <signal.h>
#include   <sys/param.h>
#include   <errno.h>
#include   <syslog.h>
#include   <sys/socket.h>
#include   <string.h>
#include   <sys/un.h>
#include   <unistd.h>
#include   <fcntl.h>
#include   <netinet/in.h>
#include   <arpa/inet.h>
#include   <errno.h>
#include   <sys/uio.h>

#ifdef SIGTSTP   /* True if BSD system */
#include  <sys/file.h>
#include  <sys/ioctl.h>
#endif

#include "ipsyslog.h"

#define PRIVPORT_SOCKET         "/dev/privport"
#define MAX_BUF_LEN             200

int privport_socket = -1;

int privbind_debug = 0;

/******************* open_privport_comm() *******************/

int open_privport_comm()
{
   struct sockaddr_un   privport_sockaddr;
   int                  privport_fn_len;

   if (privport_socket >= 0)
   {
      ip_syslog( LOG_ERR, "open_privport_comm: already open" );
      return 0;
   }

   privport_socket = socket( AF_UNIX, SOCK_STREAM, 0 );

   if (privbind_debug)
      ip_syslog( LOG_DEBUG,
         "open_privport_comm: socket(UNIX,STREAM) returns %d",privport_socket);

   if (privport_socket < 0)
   {
      ip_syslog( LOG_ERR, "open_privport_comm: socket() failed, %m" );
      privport_socket = -1;
      return 0;
   }

   privport_sockaddr.sun_family = AF_UNIX;
   strcpy( privport_sockaddr.sun_path, PRIVPORT_SOCKET );
#ifdef _SOCKADDR_LEN
   privport_sockaddr.sun_len = strlen( PRIVPORT_SOCKET ) + 1;
#endif

   if (connect( privport_socket, (struct sockaddr *) &privport_sockaddr,
             sizeof( struct sockaddr_un ) ) != 0)
   {
      ip_syslog( LOG_ERR, "open_privport_comm: connect() failed, %m" );
      close( privport_socket );
      privport_socket = -1;
      return 0;
   }
   

   if (privbind_debug)
      ip_syslog( LOG_DEBUG, "open_privport_comm: connect() OK" );

   return 1;
} /* open_privport_comm() */


/********************** get_priv_socket() ************************/

int get_priv_socket( sock_type, port_num, ip_addr )
int   sock_type;
char *port_num;
char *ip_addr;
{
   char   our_buf[80];
   int    buf_len;

   if (privport_socket < 0)
   {
      ip_syslog( LOG_ERR, "get_priv_socket: privport_socket not open" );
      return -1;
   }
   if ((sock_type != SOCK_STREAM) && (sock_type != SOCK_DGRAM))
   {
      ip_syslog( LOG_ERR, "get_priv_socket: bad value for sock_type" );
      return -1;
   }
   if (port_num == NULL)
   {
      ip_syslog( LOG_ERR, "get_priv_socket: port_num is NULL" );
      return -1;
   }
   if (strlen(port_num) > 10)
   {
      ip_syslog( LOG_ERR, "get_priv_socket: port_num string too long" );
      return -1;
   }
   if (ip_addr == NULL)
   {
      ip_syslog( LOG_ERR, "get_priv_socket: ip_addr is NULL" );
      return -1;
   }
   if (strlen(ip_addr) > 30)
   {
      ip_syslog( LOG_ERR, "get_priv_socket: ip_addr string too long" );
      return -1;
   }
   our_buf[0] = 'U';
   if (sock_type == SOCK_STREAM)   our_buf[0] = 'T';

   sprintf( our_buf + 1, " %s %s\n", port_num, ip_addr );
   buf_len = strlen( our_buf );

   if (privbind_debug)
      ip_syslog( LOG_DEBUG,
         "get_priv_socket: sending string \"%s\"", our_buf );

   if (write( privport_socket, our_buf, buf_len ) != buf_len)
   {
      ip_syslog( LOG_ERR, "get_priv_socket: write() failed" );
      perror( "get_priv_socket" );
      return -1;
   }
   else
      if (privbind_debug)
         ip_syslog( LOG_DEBUG, "get_priv_socket: write() was OK" );

   {
      int            fd, recv_rc;
      struct iovec   iov[1];
      struct msghdr  msg;
      char           recv_buf[2];

      iov[0].iov_base = (char *) (&recv_buf[0]);   /* receive one byte */
      iov[0].iov_len  = 1;

      msg.msg_iov      = iov;
      msg.msg_iovlen   = 1;
      msg.msg_name     = (caddr_t) 0;

#ifdef _SOCKADDR_LEN
      msg.msg_control       = (caddr_t) &fd;  /* address of descriptor */
      msg.msg_controllen    = sizeof(fd);     /* receive ONE descriptor */
#else
      msg.msg_accrights     = (caddr_t) &fd;  /* address of descriptor */
      msg.msg_accrightslen  = sizeof(fd);     /* receive ONE descriptor */
#endif

      recv_rc = recvmsg( privport_socket, &msg, 0);

      if (privbind_debug)
         ip_syslog( LOG_DEBUG,
                    "get_priv_socket: recvmsg() returns %d, errno=%d",
                    recv_rc, errno );

      if (recv_rc < 0)
      {
         ip_syslog( LOG_ERR, "get_priv_socket: recvmsg() failed, %m" );
         return -1;
      }
      
      if (recv_buf[0] == 'G')
      {
         if (privbind_debug)
            ip_syslog( LOG_DEBUG,
               "get_priv_socket: Returning file desc %d", fd );

         return fd;
      }
      else
      {
         ip_syslog( LOG_ERR, "get_priv_socket: Request rejected" );
         return -1;
      }
   }

   return -1;
} /* get_priv_socket() */

