/********************** mainmail.c **********************/

/******************************************************************************
 * Copyright (c) 1995,1996 Donated to the public domain                       *
 *                                                                            *
 * Original author and maintainer: Marc Chatel   (mchatel@dial.oleane.com)    *
 * Last known maintainer:          Marc Chatel   (mchatel@dial.oleane.com)    *
 *                                                                            *
 * This file was created as part of the S4 (Secure System Setup Script) kit.  *
 * Permission is granted to any person or entity to do any of the following:  *
 *       a) use this file alone or in some other software                     *
 *       b) modify this file or include parts of this file in other files     *
 *       c) re-distribute this file AS IS or modified, for non-commercial     *
 *          or commercial purposes, alone or as part of some software package *
 *                                                                            *
 * No warranties of any kind, express or implied, on the functionality and    *
 * safety of the contents of this file. Use at your own risk!                 *
 *                                                                            *
 * If you do useful changes to this file (bug fixes, portability fixes,       *
 * enhancements), you should TRY to contact the current maintainer, who may   *
 * be maintaining a "latest greatest" version of the file. You do not HAVE    *
 * TO, but you should TRY. Promote software reuse! It helps everybody,        *
 * including you!                                                             *
 ******************************************************************************/

/* mainmail: This is a privilege gateway. It is intended to be used in a
             chrooted non-privileged mail setup. mainmail's purpose is to allow
             access to the sendmail program for local users, privileged or not.

             mainmail should be installed in place of the various sendmail
             executables (or links to it) in the "main" filesystem. mainmail
             should be setuid root... mainmail should be linked statically
             to avoid eventual vulnerabilities related to shared library use!
 
             mainmail does the following:

             a) calculates the group id of the group name under which nonpriv
                sendmail operates (to be hardcoded in this source by the S4
                install script)

             b) calculates the user id of the user name under which nonpriv
                sendmail operates (to be hardcoded in this source by the S4
                install script)

             c) does a chroot to the sendmail operating directory (also
                hardcoded in this source by the S4 install script)

             d) drops privilege to the specified username, groupname

             e) rebuilds a copy of its own command-line arguments, and
                invokes "/bin/sendmail" with those arguments
*/

#include <syslog.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "drop_priv.h"

#ifdef USE_SYSLOG
#ifdef USE_PRTF
   I need either USE_SYSLOG or USE_PRTF defined, not both
#else
#endif
#else
#ifdef USE_PRTF
#else
   I need either USE_SYSLOG or USE_PRTF defined
#endif
#endif

char *called_prog_name = "/bin/sendmail";

int main( argc, argv )
int argc;
char **argv;
{
   if (!drop_priv( "S4MAILUNAME", "S4MAILGNAME", "S4MAILHOME" ))
   {
#ifdef USE_SYSLOG
      syslog( LOG_ERR, "Cannot drop privileges\n" );
#else
      fprintf( stderr, "Cannot drop privileges\n" );
#endif
      return 1;
   }

#ifdef 0
#ifdef USE_SYSLOG
      syslog( LOG_INFO, "After drop_priv" );
#else
      fprintf( stderr, "After drop_priv\n" );
#endif
#endif

   {
      char **arg_buf = NULL;
      int    our_idx = 0;

      arg_buf = (char **) malloc( sizeof(char *) * (argc + 1) );

      if (arg_buf == NULL)
      {
#ifdef USE_SYSLOG
         syslog( LOG_ERR, "Cannot allocate parameter area\n" );
#else
         fprintf( stderr, "Cannot allocate parameter area\n" );
#endif
         return 1;
      }

      arg_buf[ 0 ] = called_prog_name;

      for (our_idx = 1; our_idx < argc; our_idx++)
         arg_buf[ our_idx ] = argv[ our_idx ];

      arg_buf[ argc ] = NULL;

      (void) execv( called_prog_name, arg_buf );
   }
   return 1;
}
